# MIT License
# 
# Copyright (c) 2019 Edward D. Lee, Bryan C. Daniels
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Equations for 12-spin Ising model.

# Written on 2019/04/17.
from numpy import zeros, array, prod
from ..enumerate import mp_fast_logsumexp as fast_logsumexp
from mpmath import exp, isnan

def calc_observables(params):
    """
    Give all parameters concatenated into one array from lowest to highest order.
    Returns all correlations.
    """
    Cout = zeros((78), dtype=object)
    H = params[0:12]
    J = params[12:78]
    energyTerms = array([    +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+
    J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+
    J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+
    J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+
    J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+
    J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+
    J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+
    J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+
    J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+
    J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+
    J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+
    J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+
    J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+
    J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+
    J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+
    J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+
    J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+
    J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+
            J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65],])
    logZ = fast_logsumexp(energyTerms)[0]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1])
    Cout[0] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1])
    Cout[1] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1])
    Cout[2] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1])
    Cout[3] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1])
    Cout[4] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1])
    Cout[5] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1])
    Cout[6] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1])
    Cout[7] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1])
    Cout[8] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1])
    Cout[9] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1])
    Cout[10] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1])
    Cout[11] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[12] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[13] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[14] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[15] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[16] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[17] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[18] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[19] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1])
    Cout[20] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1])
    Cout[21] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1])
    Cout[22] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[23] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[24] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[25] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[26] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[27] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[28] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[29] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1])
    Cout[30] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1])
    Cout[31] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1])
    Cout[32] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[33] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[34] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[35] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[36] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[37] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[38] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1])
    Cout[39] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1])
    Cout[40] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1])
    Cout[41] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[42] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[43] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[44] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[45] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[46] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1])
    Cout[47] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1])
    Cout[48] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1])
    Cout[49] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[50] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[51] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[52] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[53] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1])
    Cout[54] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1])
    Cout[55] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1])
    Cout[56] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[57] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[58] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[59] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1])
    Cout[60] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1])
    Cout[61] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1])
    Cout[62] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[63] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[64] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1])
    Cout[65] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1])
    Cout[66] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1])
    Cout[67] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
 -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,
  1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1,
 -1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1])
    Cout[68] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1])
    Cout[69] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1])
    Cout[70] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1])
    Cout[71] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,
 -1,-1,-1,-1, 1, 1, 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1,
  1, 1, 1, 1,-1,-1,-1,-1,-1,-1,-1,-1, 1, 1, 1, 1])
    Cout[72] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,
 -1,-1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1,
  1, 1,-1,-1, 1, 1,-1,-1,-1,-1, 1, 1,-1,-1, 1, 1])
    Cout[73] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,
 -1, 1,-1, 1,-1, 1,-1, 1, 1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1,
  1,-1, 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1,-1, 1])
    Cout[74] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1,
  1, 1,-1,-1,-1,-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1])
    Cout[75] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1,
  1,-1, 1,-1,-1, 1,-1, 1, 1,-1, 1,-1,-1, 1,-1, 1])
    Cout[76] = exp( num[0] - logZ ) * num[1]
    num = fast_logsumexp(energyTerms, [ 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1,
  1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1, 1,-1,-1, 1])
    Cout[77] = exp( num[0] - logZ ) * num[1]
    for i in range(Cout.size):
         if isnan(Cout[i]):
             Cout[i] = 0.
    return(Cout)

def p(params):
    """
    Give all parameters concatenated into one array from lowest to highest order.
    Returns probabilities of all configurations.
    """
    Cout = zeros((78), dtype=object)
    H = params[0:12]
    J = params[12:78]
    H = params[0:12]
    J = params[12:78]
    Pout = zeros((4096), dtype=object)
    energyTerms = array([    +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+
    J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+
    J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+
    J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+
    J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+
    J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+
    J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+
    J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+
    J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+
    J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+
    J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+
    J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+
    J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], +
    H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+
    J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+
    J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+
    J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+
    J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+
    J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+
    J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+
    J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+
    H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+
    J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+
    J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+
    J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+
    J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+
    J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+
    J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+
    J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+
    J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+
    J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+
    J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+
    J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+
    J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+
    J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+
    J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+
    J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+
    J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+
    J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+
    J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+
    H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+
    J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+
    J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+
    J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+
    J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+
    J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+
    J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+
    J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+
    J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+
    J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+
    J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+
    J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+
    J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+
    J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+
    J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+
    J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+
    J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+
    J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+
    J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+
    J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+
    J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+
    J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+
    J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+
    J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+
    J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+
    J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+
    J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+
    J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+
    J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+
    J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+
    J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+
    J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+
    J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+
    J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+
    J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+
    J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+
    J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+
    J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+
    J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+
    J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+
    J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+
    J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+
    J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+
    J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+
    H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+
    J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+
    J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+
    J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+
    H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+
    J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+
    J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+
    J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+
    J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+
    J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+
    J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+
    J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+
    J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+
    J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+
    J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+
    J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+
    J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+
    J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+
    J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+
    J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+
    J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+
    J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+
    J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+
    J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+
    J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+
    J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+
    J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+
    J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+
    J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+
    J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+
    J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+
    J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+
    J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+
    J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+
    J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+
    J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+
    J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+
    J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+
    J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+
    J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+
    J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+
    J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+
    J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+
    J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+
    J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+
    J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+
    J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+
    J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]+
    H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+
    J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+
    J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+
    J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+
    J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+
    J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+
    J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+
    J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+
    J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+
    J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+
    J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+
    J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+
    J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+
    J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+
    J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+
    H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+
    J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+
    J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+
    J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+
    J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+
    J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+
    J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+
    J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+
    J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+
    J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+
    J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+
    J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+
    J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+
    J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+
    J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+
    J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+
    J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+
    J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+
    J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+
    J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+
    J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+
    J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+
    J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+
    J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+
    J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+
    J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+
    J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+
    J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+
    J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+
    J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+
    J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+
    J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+
    J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]+
    H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+
    J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+
    J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+
    J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+
    J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+
    J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+
    J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+
    H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+
    J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+
    J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+
    J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+
    J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+
    J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+
    J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+
    J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+
    J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+
    J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+
    J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+
    J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+
    J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+
    J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+
    J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+
    J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+
    J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+
    J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+
    J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+
    J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+
    J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+
    J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+
    J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+
    J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+
    H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+
    J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+
    J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+
    J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+
    J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+
    J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+
    J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+
    J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+
    J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+
    J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+
    J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+
    J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+
    J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+
    J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+
    J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+
    J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+
    J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+
    J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+
    J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+
    J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+
    J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+
    J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+
    H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+
    J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+
    J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+
    J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+
    J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+
    J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+
    J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+
    J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+
    J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+
    J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+
    J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+
    J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+
    J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+
    J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+
    J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+
    J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+
    J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+
    J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+
    J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+
    J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+
    H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+
    J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+
    J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+
    J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+
    J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+
    J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+
    J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+
    H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+
    J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+
    J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+
    J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+
    J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+
    H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+
    J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+
    J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+
    J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+
    H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+
    J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+
    J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+
    J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65], -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+
    J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+
    J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+
    J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+
            J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65],])
    logZ = fast_logsumexp(energyTerms)[0]
    Pout[0] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[5] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[6] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[7] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[8] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[9] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[10] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[11] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[12] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[13] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[14] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[15] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[16] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[17] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[18] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[19] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[20] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[21] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[22] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[23] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[24] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[25] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[26] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[27] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[28] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[29] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[30] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[31] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[32] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[33] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[34] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[35] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[36] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[37] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[38] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[39] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[40] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[41] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[42] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[43] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[44] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[45] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[46] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[47] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[48] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[49] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[50] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[51] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[52] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[53] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[54] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[55] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[56] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[57] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[58] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[59] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[60] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[61] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[62] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[63] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[64] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[65] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[66] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[67] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[68] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[69] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[70] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[71] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[72] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[73] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[74] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[75] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[76] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[77] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[78] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[79] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[80] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[81] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[82] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[83] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[84] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[85] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[86] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[87] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[88] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[89] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[90] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[91] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[92] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[93] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[94] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[95] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[96] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[97] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[98] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[99] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[100] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[101] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[102] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[103] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[104] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[105] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[106] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[107] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[108] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[109] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[110] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[111] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[112] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[113] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[114] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[115] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[116] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[117] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[118] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[119] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[120] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[121] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[122] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[123] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[124] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[125] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[126] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[127] = exp( +H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[128] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[129] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[130] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[131] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[132] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[133] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[134] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[135] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[136] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[137] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[138] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[139] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[140] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[141] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[142] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[143] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[144] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[145] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[146] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[147] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[148] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[149] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[150] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[151] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[152] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[153] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[154] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[155] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[156] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[157] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[158] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[159] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[160] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[161] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[162] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[163] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[164] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[165] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[166] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[167] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[168] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[169] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[170] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[171] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[172] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[173] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[174] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[175] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[176] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[177] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[178] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[179] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[180] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[181] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[182] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[183] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[184] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[185] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[186] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[187] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[188] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[189] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[190] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[191] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[192] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[193] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[194] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[195] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[196] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[197] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[198] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[199] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[200] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[201] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[202] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[203] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[204] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[205] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[206] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[207] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[208] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[209] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[210] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[211] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[212] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[213] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[214] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[215] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[216] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[217] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[218] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[219] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[220] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[221] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[222] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[223] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[224] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[225] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[226] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[227] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[228] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[229] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[230] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[231] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[232] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[233] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[234] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[235] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[236] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[237] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[238] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[239] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[240] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[241] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[242] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[243] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[244] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[245] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[246] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[247] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[248] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[249] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[250] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[251] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[252] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[253] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[254] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[255] = exp( +H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[256] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[257] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[258] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[259] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[260] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[261] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[262] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[263] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[264] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[265] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[266] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[267] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[268] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[269] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[270] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[271] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[272] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[273] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[274] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[275] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[276] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[277] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[278] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[279] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[280] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[281] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[282] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[283] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[284] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[285] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[286] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[287] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[288] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[289] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[290] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[291] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[292] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[293] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[294] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[295] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[296] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[297] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[298] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[299] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[300] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[301] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[302] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[303] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[304] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[305] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[306] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[307] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[308] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[309] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[310] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[311] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[312] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[313] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[314] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[315] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[316] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[317] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[318] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[319] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[320] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[321] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[322] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[323] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[324] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[325] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[326] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[327] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[328] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[329] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[330] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[331] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[332] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[333] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[334] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[335] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[336] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[337] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[338] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[339] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[340] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[341] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[342] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[343] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[344] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[345] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[346] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[347] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[348] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[349] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[350] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[351] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[352] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[353] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[354] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[355] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[356] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[357] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[358] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[359] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[360] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[361] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[362] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[363] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[364] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[365] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[366] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[367] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[368] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[369] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[370] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[371] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[372] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[373] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[374] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[375] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[376] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[377] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[378] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[379] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[380] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[381] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[382] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[383] = exp( +H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[384] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[385] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[386] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[387] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[388] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[389] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[390] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[391] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[392] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[393] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[394] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[395] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[396] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[397] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[398] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[399] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[400] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[401] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[402] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[403] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[404] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[405] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[406] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[407] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[408] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[409] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[410] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[411] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[412] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[413] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[414] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[415] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[416] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[417] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[418] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[419] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[420] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[421] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[422] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[423] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[424] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[425] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[426] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[427] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[428] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[429] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[430] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[431] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[432] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[433] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[434] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[435] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[436] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[437] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[438] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[439] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[440] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[441] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[442] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[443] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[444] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[445] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[446] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[447] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[448] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[449] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[450] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[451] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[452] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[453] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[454] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[455] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[456] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[457] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[458] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[459] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[460] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[461] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[462] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[463] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[464] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[465] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[466] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[467] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[468] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[469] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[470] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[471] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[472] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[473] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[474] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[475] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[476] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[477] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[478] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[479] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[480] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[481] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[482] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[483] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[484] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[485] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[486] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[487] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[488] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[489] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[490] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[491] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[492] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[493] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[494] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[495] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[496] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[497] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[498] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[499] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[500] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[501] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[502] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[503] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[504] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[505] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[506] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[507] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[508] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[509] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[510] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[511] = exp( +H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[512] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[513] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[514] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[515] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[516] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[517] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[518] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[519] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[520] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[521] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[522] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[523] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[524] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[525] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[526] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[527] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[528] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[529] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[530] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[531] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[532] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[533] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[534] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[535] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[536] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[537] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[538] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[539] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[540] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[541] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[542] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[543] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[544] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[545] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[546] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[547] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[548] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[549] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[550] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[551] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[552] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[553] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[554] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[555] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[556] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[557] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[558] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[559] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[560] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[561] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[562] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[563] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[564] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[565] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[566] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[567] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[568] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[569] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[570] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[571] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[572] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[573] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[574] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[575] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[576] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[577] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[578] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[579] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[580] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[581] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[582] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[583] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[584] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[585] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[586] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[587] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[588] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[589] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[590] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[591] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[592] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[593] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[594] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[595] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[596] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[597] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[598] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[599] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[600] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[601] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[602] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[603] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[604] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[605] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[606] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[607] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[608] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[609] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[610] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[611] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[612] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[613] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[614] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[615] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[616] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[617] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[618] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[619] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[620] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[621] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[622] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[623] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[624] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[625] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[626] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[627] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[628] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[629] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[630] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[631] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[632] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[633] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[634] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[635] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[636] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[637] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[638] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[639] = exp( +H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[640] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[641] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[642] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[643] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[644] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[645] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[646] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[647] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[648] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[649] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[650] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[651] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[652] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[653] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[654] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[655] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[656] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[657] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[658] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[659] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[660] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[661] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[662] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[663] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[664] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[665] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[666] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[667] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[668] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[669] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[670] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[671] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[672] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[673] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[674] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[675] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[676] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[677] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[678] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[679] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[680] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[681] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[682] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[683] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[684] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[685] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[686] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[687] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[688] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[689] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[690] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[691] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[692] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[693] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[694] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[695] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[696] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[697] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[698] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[699] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[700] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[701] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[702] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[703] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[704] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[705] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[706] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[707] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[708] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[709] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[710] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[711] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[712] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[713] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[714] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[715] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[716] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[717] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[718] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[719] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[720] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[721] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[722] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[723] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[724] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[725] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[726] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[727] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[728] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[729] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[730] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[731] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[732] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[733] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[734] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[735] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[736] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[737] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[738] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[739] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[740] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[741] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[742] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[743] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[744] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[745] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[746] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[747] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[748] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[749] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[750] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[751] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[752] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[753] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[754] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[755] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[756] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[757] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[758] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[759] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[760] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[761] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[762] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[763] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[764] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[765] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[766] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[767] = exp( +H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[768] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[769] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[770] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[771] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[772] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[773] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[774] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[775] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[776] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[777] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[778] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[779] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[780] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[781] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[782] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[783] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[784] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[785] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[786] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[787] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[788] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[789] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[790] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[791] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[792] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[793] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[794] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[795] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[796] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[797] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[798] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[799] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[800] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[801] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[802] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[803] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[804] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[805] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[806] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[807] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[808] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[809] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[810] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[811] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[812] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[813] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[814] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[815] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[816] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[817] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[818] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[819] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[820] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[821] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[822] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[823] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[824] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[825] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[826] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[827] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[828] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[829] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[830] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[831] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[832] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[833] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[834] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[835] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[836] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[837] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[838] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[839] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[840] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[841] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[842] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[843] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[844] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[845] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[846] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[847] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[848] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[849] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[850] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[851] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[852] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[853] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[854] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[855] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[856] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[857] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[858] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[859] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[860] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[861] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[862] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[863] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[864] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[865] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[866] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[867] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[868] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[869] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[870] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[871] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[872] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[873] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[874] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[875] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[876] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[877] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[878] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[879] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[880] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[881] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[882] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[883] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[884] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[885] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[886] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[887] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[888] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[889] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[890] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[891] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[892] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[893] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[894] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[895] = exp( +H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[896] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[897] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[898] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[899] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[900] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[901] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[902] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[903] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[904] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[905] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[906] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[907] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[908] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[909] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[910] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[911] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[912] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[913] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[914] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[915] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[916] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[917] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[918] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[919] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[920] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[921] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[922] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[923] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[924] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[925] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[926] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[927] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[928] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[929] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[930] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[931] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[932] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[933] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[934] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[935] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[936] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[937] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[938] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[939] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[940] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[941] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[942] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[943] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[944] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[945] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[946] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[947] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[948] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[949] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[950] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[951] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[952] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[953] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[954] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[955] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[956] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[957] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[958] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[959] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[960] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[961] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[962] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[963] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[964] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[965] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[966] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[967] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[968] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[969] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[970] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[971] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[972] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[973] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[974] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[975] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[976] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[977] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[978] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[979] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[980] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[981] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[982] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[983] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[984] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[985] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[986] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[987] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[988] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[989] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[990] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[991] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[992] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[993] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[994] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[995] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[996] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[997] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[998] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[999] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1000] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1001] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1002] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1003] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1004] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1005] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1006] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1007] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1008] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1009] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1010] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1011] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1012] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1013] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1014] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1015] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1016] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1017] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1018] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1019] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1020] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1021] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1022] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1023] = exp( +H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1024] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1025] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1026] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1027] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1028] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1029] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1030] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1031] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1032] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1033] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1034] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1035] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1036] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1037] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1038] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1039] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1040] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1041] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1042] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1043] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1044] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1045] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1046] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1047] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1048] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1049] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1050] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1051] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1052] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1053] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1054] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1055] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1056] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1057] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1058] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1059] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1060] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1061] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1062] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1063] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1064] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1065] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1066] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1067] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1068] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1069] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1070] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1071] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1072] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1073] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1074] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1075] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1076] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1077] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1078] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1079] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1080] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1081] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1082] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1083] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1084] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1085] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1086] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1087] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1088] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1089] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1090] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1091] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1092] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1093] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1094] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1095] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1096] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1097] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1098] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1099] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1100] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1101] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1102] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1103] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1104] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1105] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1106] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1107] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1108] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1109] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1110] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1111] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1112] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1113] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1114] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1115] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1116] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1117] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1118] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1119] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1120] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1121] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1122] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1123] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1124] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1125] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1126] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1127] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1128] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1129] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1130] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1131] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1132] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1133] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1134] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1135] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1136] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1137] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1138] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1139] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1140] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1141] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1142] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1143] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1144] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1145] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1146] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1147] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1148] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1149] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1150] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1151] = exp( +H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1152] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1153] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1154] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1155] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1156] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1157] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1158] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1159] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1160] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1161] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1162] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1163] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1164] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1165] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1166] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1167] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1168] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1169] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1170] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1171] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1172] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1173] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1174] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1175] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1176] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1177] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1178] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1179] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1180] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1181] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1182] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1183] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1184] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1185] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1186] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1187] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1188] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1189] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1190] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1191] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1192] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1193] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1194] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1195] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1196] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1197] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1198] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1199] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1200] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1201] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1202] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1203] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1204] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1205] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1206] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1207] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1208] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1209] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1210] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1211] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1212] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1213] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1214] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1215] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1216] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1217] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1218] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1219] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1220] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1221] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1222] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1223] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1224] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1225] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1226] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1227] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1228] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1229] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1230] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1231] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1232] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1233] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1234] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1235] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1236] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1237] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1238] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1239] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1240] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1241] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1242] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1243] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1244] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1245] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1246] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1247] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1248] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1249] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1250] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1251] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1252] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1253] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1254] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1255] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1256] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1257] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1258] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1259] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1260] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1261] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1262] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1263] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1264] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1265] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1266] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1267] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1268] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1269] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1270] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1271] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1272] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1273] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1274] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1275] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1276] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1277] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1278] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1279] = exp( +H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1280] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1281] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1282] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1283] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1284] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1285] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1286] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1287] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1288] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1289] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1290] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1291] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1292] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1293] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1294] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1295] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1296] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1297] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1298] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1299] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1300] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1301] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1302] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1303] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1304] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1305] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1306] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1307] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1308] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1309] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1310] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1311] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1312] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1313] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1314] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1315] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1316] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1317] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1318] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1319] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1320] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1321] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1322] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1323] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1324] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1325] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1326] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1327] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1328] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1329] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1330] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1331] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1332] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1333] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1334] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1335] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1336] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1337] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1338] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1339] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1340] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1341] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1342] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1343] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1344] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1345] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1346] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1347] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1348] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1349] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1350] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1351] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1352] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1353] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1354] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1355] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1356] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1357] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1358] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1359] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1360] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1361] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1362] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1363] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1364] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1365] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1366] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1367] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1368] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1369] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1370] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1371] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1372] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1373] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1374] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1375] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1376] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1377] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1378] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1379] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1380] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1381] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1382] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1383] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1384] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1385] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1386] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1387] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1388] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1389] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1390] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1391] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1392] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1393] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1394] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1395] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1396] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1397] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1398] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1399] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1400] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1401] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1402] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1403] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1404] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1405] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1406] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1407] = exp( +H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1408] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1409] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1410] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1411] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1412] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1413] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1414] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1415] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1416] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1417] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1418] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1419] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1420] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1421] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1422] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1423] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1424] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1425] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1426] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1427] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1428] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1429] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1430] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1431] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1432] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1433] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1434] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1435] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1436] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1437] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1438] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1439] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1440] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1441] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1442] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1443] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1444] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1445] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1446] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1447] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1448] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1449] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1450] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1451] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1452] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1453] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1454] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1455] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1456] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1457] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1458] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1459] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1460] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1461] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1462] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1463] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1464] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1465] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1466] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1467] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1468] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1469] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1470] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1471] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1472] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1473] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1474] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1475] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1476] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1477] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1478] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1479] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1480] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1481] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1482] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1483] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1484] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1485] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1486] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1487] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1488] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1489] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1490] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1491] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1492] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1493] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1494] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1495] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1496] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1497] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1498] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1499] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1500] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1501] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1502] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1503] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1504] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1505] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1506] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1507] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1508] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1509] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1510] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1511] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1512] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1513] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1514] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1515] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1516] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1517] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1518] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1519] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1520] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1521] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1522] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1523] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1524] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1525] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1526] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1527] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1528] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1529] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1530] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1531] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1532] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1533] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1534] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1535] = exp( +H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1536] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1537] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1538] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1539] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1540] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1541] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1542] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1543] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1544] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1545] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1546] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1547] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1548] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1549] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1550] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1551] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1552] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1553] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1554] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1555] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1556] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1557] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1558] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1559] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1560] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1561] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1562] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1563] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1564] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1565] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1566] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1567] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1568] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1569] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1570] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1571] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1572] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1573] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1574] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1575] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1576] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1577] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1578] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1579] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1580] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1581] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1582] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1583] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1584] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1585] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1586] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1587] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1588] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1589] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1590] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1591] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1592] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1593] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1594] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1595] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1596] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1597] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1598] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1599] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1600] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1601] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1602] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1603] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1604] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1605] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1606] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1607] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1608] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1609] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1610] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1611] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1612] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1613] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1614] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1615] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1616] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1617] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1618] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1619] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1620] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1621] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1622] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1623] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1624] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1625] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1626] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1627] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1628] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1629] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1630] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1631] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1632] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1633] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1634] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1635] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1636] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1637] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1638] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1639] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1640] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1641] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1642] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1643] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1644] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1645] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1646] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1647] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1648] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1649] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1650] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1651] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1652] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1653] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1654] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1655] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1656] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1657] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1658] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1659] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1660] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1661] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1662] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1663] = exp( +H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1664] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1665] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1666] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1667] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1668] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1669] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1670] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1671] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1672] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1673] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1674] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1675] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1676] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1677] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1678] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1679] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1680] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1681] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1682] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1683] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1684] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1685] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1686] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1687] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1688] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1689] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1690] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1691] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1692] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1693] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1694] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1695] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1696] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1697] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1698] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1699] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1700] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1701] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1702] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1703] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1704] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1705] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1706] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1707] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1708] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1709] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1710] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1711] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1712] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1713] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1714] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1715] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1716] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1717] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1718] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1719] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1720] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1721] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1722] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1723] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1724] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1725] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1726] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1727] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1728] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1729] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1730] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1731] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1732] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1733] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1734] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1735] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1736] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1737] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1738] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1739] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1740] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1741] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1742] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1743] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1744] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1745] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1746] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1747] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1748] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1749] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1750] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1751] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1752] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1753] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1754] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1755] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1756] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1757] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1758] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1759] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1760] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1761] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1762] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1763] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1764] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1765] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1766] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1767] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1768] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1769] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1770] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1771] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1772] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1773] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1774] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1775] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1776] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1777] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1778] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1779] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1780] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1781] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1782] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1783] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1784] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1785] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1786] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1787] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1788] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1789] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1790] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1791] = exp( +H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1792] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1793] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1794] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1795] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1796] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1797] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1798] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1799] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1800] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1801] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1802] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1803] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1804] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1805] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1806] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1807] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1808] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1809] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1810] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1811] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1812] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1813] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1814] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1815] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1816] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1817] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1818] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1819] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1820] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1821] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1822] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1823] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1824] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1825] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1826] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1827] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1828] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1829] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1830] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1831] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1832] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1833] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1834] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1835] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1836] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1837] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1838] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1839] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1840] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1841] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1842] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1843] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1844] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1845] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1846] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1847] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1848] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1849] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1850] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1851] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1852] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1853] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1854] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1855] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1856] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1857] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1858] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1859] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1860] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1861] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1862] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1863] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1864] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1865] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1866] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1867] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1868] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1869] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1870] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1871] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1872] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1873] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1874] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1875] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1876] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1877] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1878] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1879] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1880] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1881] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1882] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1883] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1884] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1885] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1886] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1887] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1888] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1889] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1890] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1891] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1892] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1893] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1894] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1895] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1896] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1897] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1898] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1899] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1900] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1901] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1902] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1903] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1904] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1905] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1906] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1907] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1908] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1909] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1910] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1911] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1912] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1913] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1914] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1915] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1916] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1917] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1918] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1919] = exp( +H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1920] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1921] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1922] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1923] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1924] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1925] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1926] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1927] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1928] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1929] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1930] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1931] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1932] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1933] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1934] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1935] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1936] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1937] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1938] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1939] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1940] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1941] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1942] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1943] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1944] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1945] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1946] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1947] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1948] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1949] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1950] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1951] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1952] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1953] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1954] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1955] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1956] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1957] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1958] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1959] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1960] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1961] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1962] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1963] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1964] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1965] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1966] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1967] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1968] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1969] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1970] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1971] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1972] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1973] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1974] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1975] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1976] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1977] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1978] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1979] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1980] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1981] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1982] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1983] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1984] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1985] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1986] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1987] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1988] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1989] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1990] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1991] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1992] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[1993] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1994] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1995] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1996] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[1997] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[1998] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[1999] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2000] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2001] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2002] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2003] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2004] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2005] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2006] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2007] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2008] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2009] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2010] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2011] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2012] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2013] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2014] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2015] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2016] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2017] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2018] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2019] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2020] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2021] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2022] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2023] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2024] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2025] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2026] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2027] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2028] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2029] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2030] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2031] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2032] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2033] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2034] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2035] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2036] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2037] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2038] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2039] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2040] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2041] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2042] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2043] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2044] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2045] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2046] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2047] = exp( +H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2048] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2049] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2050] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2051] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2052] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2053] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2054] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2055] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2056] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2057] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2058] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2059] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2060] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2061] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2062] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2063] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2064] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2065] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2066] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2067] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2068] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2069] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2070] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2071] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2072] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2073] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2074] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2075] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2076] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2077] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2078] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2079] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2080] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2081] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2082] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2083] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2084] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2085] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2086] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2087] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2088] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2089] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2090] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2091] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2092] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2093] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2094] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2095] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2096] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2097] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2098] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2099] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2100] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2101] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2102] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2103] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2104] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2105] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2106] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2107] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2108] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2109] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2110] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2111] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2112] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2113] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2114] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2115] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2116] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2117] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2118] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2119] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2120] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2121] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2122] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2123] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2124] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2125] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2126] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2127] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2128] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2129] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2130] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2131] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2132] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2133] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2134] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2135] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2136] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2137] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2138] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2139] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2140] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2141] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2142] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2143] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2144] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2145] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2146] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2147] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2148] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2149] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2150] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2151] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2152] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2153] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2154] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2155] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2156] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2157] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2158] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2159] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2160] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2161] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2162] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2163] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2164] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2165] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2166] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2167] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2168] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2169] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2170] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2171] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2172] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2173] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2174] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2175] = exp( -H[0]+H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2176] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2177] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2178] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2179] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2180] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2181] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2182] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2183] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2184] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2185] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2186] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2187] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2188] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2189] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2190] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2191] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2192] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2193] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2194] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2195] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2196] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2197] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2198] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2199] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2200] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2201] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2202] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2203] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2204] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2205] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2206] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2207] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2208] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2209] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2210] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2211] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2212] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2213] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2214] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2215] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2216] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2217] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2218] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2219] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2220] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2221] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2222] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2223] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2224] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2225] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2226] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2227] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2228] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2229] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2230] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2231] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2232] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2233] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2234] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2235] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2236] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2237] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2238] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2239] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2240] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2241] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2242] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2243] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2244] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2245] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2246] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2247] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2248] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2249] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2250] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2251] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2252] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2253] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2254] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2255] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2256] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2257] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2258] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2259] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2260] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2261] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2262] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2263] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2264] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2265] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2266] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2267] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2268] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2269] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2270] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2271] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2272] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2273] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2274] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2275] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2276] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2277] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2278] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2279] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2280] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2281] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2282] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2283] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2284] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2285] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2286] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2287] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2288] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2289] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2290] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2291] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2292] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2293] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2294] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2295] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2296] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2297] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2298] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2299] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2300] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2301] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2302] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2303] = exp( -H[0]+H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2304] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2305] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2306] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2307] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2308] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2309] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2310] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2311] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2312] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2313] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2314] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2315] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2316] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2317] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2318] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2319] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2320] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2321] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2322] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2323] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2324] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2325] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2326] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2327] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2328] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2329] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2330] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2331] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2332] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2333] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2334] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2335] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2336] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2337] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2338] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2339] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2340] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2341] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2342] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2343] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2344] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2345] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2346] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2347] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2348] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2349] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2350] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2351] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2352] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2353] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2354] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2355] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2356] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2357] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2358] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2359] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2360] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2361] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2362] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2363] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2364] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2365] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2366] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2367] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2368] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2369] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2370] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2371] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2372] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2373] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2374] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2375] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2376] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2377] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2378] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2379] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2380] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2381] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2382] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2383] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2384] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2385] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2386] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2387] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2388] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2389] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2390] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2391] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2392] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2393] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2394] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2395] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2396] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2397] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2398] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2399] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2400] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2401] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2402] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2403] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2404] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2405] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2406] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2407] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2408] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2409] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2410] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2411] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2412] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2413] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2414] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2415] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2416] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2417] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2418] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2419] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2420] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2421] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2422] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2423] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2424] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2425] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2426] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2427] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2428] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2429] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2430] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2431] = exp( -H[0]+H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2432] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2433] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2434] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2435] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2436] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2437] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2438] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2439] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2440] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2441] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2442] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2443] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2444] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2445] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2446] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2447] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2448] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2449] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2450] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2451] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2452] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2453] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2454] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2455] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2456] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2457] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2458] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2459] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2460] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2461] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2462] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2463] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2464] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2465] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2466] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2467] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2468] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2469] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2470] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2471] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2472] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2473] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2474] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2475] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2476] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2477] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2478] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2479] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2480] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2481] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2482] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2483] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2484] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2485] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2486] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2487] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2488] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2489] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2490] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2491] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2492] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2493] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2494] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2495] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2496] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2497] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2498] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2499] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2500] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2501] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2502] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2503] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2504] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2505] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2506] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2507] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2508] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2509] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2510] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2511] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2512] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2513] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2514] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2515] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2516] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2517] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2518] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2519] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2520] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2521] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2522] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2523] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2524] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2525] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2526] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2527] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2528] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2529] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2530] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2531] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2532] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2533] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2534] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2535] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2536] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2537] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2538] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2539] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2540] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2541] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2542] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2543] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2544] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2545] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2546] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2547] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2548] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2549] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2550] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2551] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2552] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2553] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2554] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2555] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2556] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2557] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2558] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2559] = exp( -H[0]+H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2560] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2561] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2562] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2563] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2564] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2565] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2566] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2567] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2568] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2569] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2570] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2571] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2572] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2573] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2574] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2575] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2576] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2577] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2578] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2579] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2580] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2581] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2582] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2583] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2584] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2585] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2586] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2587] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2588] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2589] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2590] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2591] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2592] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2593] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2594] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2595] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2596] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2597] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2598] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2599] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2600] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2601] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2602] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2603] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2604] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2605] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2606] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2607] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2608] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2609] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2610] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2611] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2612] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2613] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2614] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2615] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2616] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2617] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2618] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2619] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2620] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2621] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2622] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2623] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2624] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2625] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2626] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2627] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2628] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2629] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2630] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2631] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2632] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2633] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2634] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2635] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2636] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2637] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2638] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2639] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2640] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2641] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2642] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2643] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2644] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2645] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2646] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2647] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2648] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2649] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2650] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2651] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2652] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2653] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2654] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2655] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2656] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2657] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2658] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2659] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2660] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2661] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2662] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2663] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2664] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2665] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2666] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2667] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2668] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2669] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2670] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2671] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2672] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2673] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2674] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2675] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2676] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2677] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2678] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2679] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2680] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2681] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2682] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2683] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2684] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2685] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2686] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2687] = exp( -H[0]+H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2688] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2689] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2690] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2691] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2692] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2693] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2694] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2695] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2696] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2697] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2698] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2699] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2700] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2701] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2702] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2703] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2704] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2705] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2706] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2707] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2708] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2709] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2710] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2711] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2712] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2713] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2714] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2715] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2716] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2717] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2718] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2719] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2720] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2721] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2722] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2723] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2724] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2725] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2726] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2727] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2728] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2729] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2730] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2731] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2732] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2733] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2734] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2735] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2736] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2737] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2738] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2739] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2740] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2741] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2742] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2743] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2744] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2745] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2746] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2747] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2748] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2749] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2750] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2751] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2752] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2753] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2754] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2755] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2756] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2757] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2758] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2759] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2760] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2761] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2762] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2763] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2764] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2765] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2766] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2767] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2768] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2769] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2770] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2771] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2772] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2773] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2774] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2775] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2776] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2777] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2778] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2779] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2780] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2781] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2782] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2783] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2784] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2785] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2786] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2787] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2788] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2789] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2790] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2791] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2792] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2793] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2794] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2795] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2796] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2797] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2798] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2799] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2800] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2801] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2802] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2803] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2804] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2805] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2806] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2807] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2808] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2809] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2810] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2811] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2812] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2813] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2814] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2815] = exp( -H[0]+H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2816] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2817] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2818] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2819] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2820] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2821] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2822] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2823] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2824] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2825] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2826] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2827] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2828] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2829] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2830] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2831] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2832] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2833] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2834] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2835] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2836] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2837] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2838] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2839] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2840] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2841] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2842] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2843] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2844] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2845] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2846] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2847] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2848] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2849] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2850] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2851] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2852] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2853] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2854] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2855] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2856] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2857] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2858] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2859] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2860] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2861] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2862] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2863] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2864] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2865] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2866] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2867] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2868] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2869] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2870] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2871] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2872] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2873] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2874] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2875] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2876] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2877] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2878] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2879] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2880] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2881] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2882] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2883] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2884] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2885] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2886] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2887] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2888] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2889] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2890] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2891] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2892] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2893] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2894] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2895] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2896] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2897] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2898] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2899] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2900] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2901] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2902] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2903] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2904] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2905] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2906] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2907] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2908] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2909] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2910] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2911] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2912] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2913] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2914] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2915] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2916] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2917] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2918] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2919] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2920] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2921] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2922] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2923] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2924] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2925] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2926] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2927] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2928] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2929] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2930] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2931] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2932] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2933] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2934] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2935] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2936] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2937] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2938] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2939] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2940] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2941] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2942] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2943] = exp( -H[0]+H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2944] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2945] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2946] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2947] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2948] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2949] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2950] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2951] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2952] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2953] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2954] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2955] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2956] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2957] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2958] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2959] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2960] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2961] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2962] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2963] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2964] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2965] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2966] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2967] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2968] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2969] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2970] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2971] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2972] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2973] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2974] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2975] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2976] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2977] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2978] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2979] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2980] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2981] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2982] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2983] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2984] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2985] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2986] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2987] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2988] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2989] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2990] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2991] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2992] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[2993] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2994] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2995] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2996] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[2997] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[2998] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[2999] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3000] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3001] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3002] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3003] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3004] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3005] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3006] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3007] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3008] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3009] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3010] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3011] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3012] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3013] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3014] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3015] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3016] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3017] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3018] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3019] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3020] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3021] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3022] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3023] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3024] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3025] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3026] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3027] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3028] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3029] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3030] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3031] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3032] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3033] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3034] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3035] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3036] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3037] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3038] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3039] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3040] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3041] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3042] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3043] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3044] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3045] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3046] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3047] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3048] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3049] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3050] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3051] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3052] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3053] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3054] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3055] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3056] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3057] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3058] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3059] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3060] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3061] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3062] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3063] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3064] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3065] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3066] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3067] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3068] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3069] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3070] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3071] = exp( -H[0]+H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]-J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3072] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3073] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3074] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3075] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3076] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3077] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3078] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3079] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3080] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3081] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3082] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3083] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3084] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3085] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3086] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3087] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3088] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3089] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3090] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3091] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3092] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3093] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3094] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3095] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3096] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3097] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3098] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3099] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3100] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3101] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3102] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3103] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3104] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3105] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3106] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3107] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3108] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3109] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3110] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3111] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3112] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3113] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3114] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3115] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3116] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3117] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3118] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3119] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3120] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3121] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3122] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3123] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3124] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3125] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3126] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3127] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3128] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3129] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3130] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3131] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3132] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3133] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3134] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3135] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3136] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3137] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3138] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3139] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3140] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3141] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3142] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3143] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3144] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3145] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3146] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3147] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3148] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3149] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3150] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3151] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3152] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3153] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3154] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3155] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3156] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3157] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3158] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3159] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3160] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3161] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3162] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3163] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3164] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3165] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3166] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3167] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3168] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3169] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3170] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3171] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3172] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3173] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3174] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3175] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3176] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3177] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3178] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3179] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3180] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3181] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3182] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3183] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3184] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3185] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3186] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3187] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3188] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3189] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3190] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3191] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3192] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3193] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3194] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3195] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3196] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3197] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3198] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3199] = exp( -H[0]-H[1]+H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3200] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3201] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3202] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3203] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3204] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3205] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3206] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3207] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3208] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3209] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3210] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3211] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3212] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3213] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3214] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3215] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3216] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3217] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3218] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3219] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3220] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3221] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3222] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3223] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3224] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3225] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3226] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3227] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3228] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3229] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3230] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3231] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3232] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3233] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3234] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3235] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3236] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3237] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3238] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3239] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3240] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3241] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3242] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3243] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3244] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3245] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3246] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3247] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3248] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3249] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3250] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3251] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3252] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3253] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3254] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3255] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3256] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3257] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3258] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3259] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3260] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3261] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3262] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3263] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3264] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3265] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3266] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3267] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3268] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3269] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3270] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3271] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3272] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3273] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3274] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3275] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3276] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3277] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3278] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3279] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3280] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3281] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3282] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3283] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3284] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3285] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3286] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3287] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3288] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3289] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3290] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3291] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3292] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3293] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3294] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3295] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3296] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3297] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3298] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3299] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3300] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3301] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3302] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3303] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3304] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3305] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3306] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3307] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3308] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3309] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3310] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3311] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3312] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3313] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3314] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3315] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3316] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3317] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3318] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3319] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3320] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3321] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3322] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3323] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3324] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3325] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3326] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3327] = exp( -H[0]-H[1]+H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3328] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3329] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3330] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3331] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3332] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3333] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3334] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3335] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3336] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3337] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3338] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3339] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3340] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3341] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3342] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3343] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3344] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3345] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3346] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3347] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3348] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3349] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3350] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3351] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3352] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3353] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3354] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3355] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3356] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3357] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3358] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3359] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3360] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3361] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3362] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3363] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3364] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3365] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3366] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3367] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3368] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3369] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3370] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3371] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3372] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3373] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3374] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3375] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3376] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3377] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3378] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3379] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3380] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3381] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3382] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3383] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3384] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3385] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3386] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3387] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3388] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3389] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3390] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3391] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3392] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3393] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3394] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3395] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3396] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3397] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3398] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3399] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3400] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3401] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3402] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3403] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3404] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3405] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3406] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3407] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3408] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3409] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3410] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3411] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3412] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3413] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3414] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3415] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3416] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3417] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3418] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3419] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3420] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3421] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3422] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3423] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3424] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3425] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3426] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3427] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3428] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3429] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3430] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3431] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3432] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3433] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3434] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3435] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3436] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3437] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3438] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3439] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3440] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3441] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3442] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3443] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3444] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3445] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3446] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3447] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3448] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3449] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3450] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3451] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3452] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3453] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3454] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3455] = exp( -H[0]-H[1]+H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3456] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3457] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3458] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3459] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3460] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3461] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3462] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3463] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3464] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3465] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3466] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3467] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3468] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3469] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3470] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3471] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3472] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3473] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3474] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3475] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3476] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3477] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3478] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3479] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3480] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3481] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3482] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3483] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3484] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3485] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3486] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3487] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3488] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3489] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3490] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3491] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3492] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3493] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3494] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3495] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3496] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3497] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3498] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3499] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3500] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3501] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3502] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3503] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3504] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3505] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3506] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3507] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3508] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3509] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3510] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3511] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3512] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3513] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3514] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3515] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3516] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3517] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3518] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3519] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3520] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3521] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3522] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3523] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3524] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3525] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3526] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3527] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3528] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3529] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3530] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3531] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3532] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3533] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3534] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3535] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3536] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3537] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3538] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3539] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3540] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3541] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3542] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3543] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3544] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3545] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3546] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3547] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3548] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3549] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3550] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3551] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3552] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3553] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3554] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3555] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3556] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3557] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3558] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3559] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3560] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3561] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3562] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3563] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3564] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3565] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3566] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3567] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3568] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3569] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3570] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3571] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3572] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3573] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3574] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3575] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3576] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3577] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3578] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3579] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3580] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3581] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3582] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3583] = exp( -H[0]-H[1]+H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]-J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]-J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3584] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3585] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3586] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3587] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3588] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3589] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3590] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3591] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3592] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3593] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3594] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3595] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3596] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3597] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3598] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3599] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3600] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3601] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3602] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3603] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3604] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3605] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3606] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3607] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3608] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3609] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3610] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3611] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3612] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3613] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3614] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3615] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3616] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3617] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3618] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3619] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3620] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3621] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3622] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3623] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3624] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3625] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3626] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3627] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3628] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3629] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3630] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3631] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3632] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3633] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3634] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3635] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3636] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3637] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3638] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3639] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3640] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3641] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3642] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3643] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3644] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3645] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3646] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3647] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3648] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3649] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3650] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3651] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3652] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3653] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3654] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3655] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3656] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3657] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3658] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3659] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3660] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3661] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3662] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3663] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3664] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3665] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3666] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3667] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3668] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3669] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3670] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3671] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3672] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3673] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3674] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3675] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3676] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3677] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3678] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3679] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3680] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3681] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3682] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3683] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3684] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3685] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3686] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3687] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3688] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3689] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3690] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3691] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3692] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3693] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3694] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3695] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3696] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3697] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3698] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3699] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3700] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3701] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3702] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3703] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3704] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3705] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3706] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3707] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3708] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3709] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3710] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3711] = exp( -H[0]-H[1]-H[2]+H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3712] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3713] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3714] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3715] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3716] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3717] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3718] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3719] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3720] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3721] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3722] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3723] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3724] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3725] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3726] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3727] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3728] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3729] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3730] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3731] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3732] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3733] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3734] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3735] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3736] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3737] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3738] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3739] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3740] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3741] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3742] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3743] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3744] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3745] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3746] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3747] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3748] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3749] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3750] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3751] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3752] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3753] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3754] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3755] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3756] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3757] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3758] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3759] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3760] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3761] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3762] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3763] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3764] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3765] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3766] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3767] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3768] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3769] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3770] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3771] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3772] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3773] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3774] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3775] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3776] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3777] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3778] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3779] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3780] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3781] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3782] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3783] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3784] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3785] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3786] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3787] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3788] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3789] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3790] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3791] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3792] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3793] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3794] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3795] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3796] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3797] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3798] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3799] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3800] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3801] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3802] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3803] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3804] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3805] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3806] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3807] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3808] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3809] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3810] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3811] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3812] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3813] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3814] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3815] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3816] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3817] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3818] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3819] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3820] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3821] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3822] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3823] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3824] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3825] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3826] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3827] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3828] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3829] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3830] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3831] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3832] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3833] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3834] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3835] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3836] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3837] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3838] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3839] = exp( -H[0]-H[1]-H[2]+H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]-J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]-J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]-J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3840] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3841] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3842] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3843] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3844] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3845] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3846] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3847] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3848] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3849] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3850] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3851] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3852] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3853] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3854] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3855] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3856] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3857] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3858] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3859] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3860] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3861] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3862] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3863] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3864] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3865] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3866] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3867] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3868] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3869] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3870] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3871] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3872] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3873] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3874] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3875] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3876] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3877] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3878] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3879] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3880] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3881] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3882] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3883] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3884] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3885] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3886] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3887] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3888] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3889] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3890] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3891] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3892] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3893] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3894] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3895] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3896] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3897] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3898] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3899] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3900] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3901] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3902] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3903] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3904] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3905] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3906] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3907] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3908] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3909] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3910] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3911] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3912] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3913] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3914] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3915] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3916] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3917] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3918] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3919] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3920] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3921] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3922] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3923] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3924] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3925] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3926] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3927] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3928] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3929] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3930] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3931] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3932] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3933] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3934] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3935] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3936] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3937] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3938] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3939] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3940] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3941] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3942] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3943] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3944] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3945] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3946] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3947] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3948] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3949] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3950] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3951] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3952] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3953] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3954] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3955] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3956] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3957] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3958] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3959] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3960] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3961] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3962] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3963] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3964] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3965] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3966] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3967] = exp( -H[0]-H[1]-H[2]-H[3]+H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]-J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]-J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]-J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]-J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3968] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3969] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3970] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3971] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3972] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3973] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3974] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3975] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3976] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3977] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3978] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3979] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3980] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3981] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3982] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3983] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3984] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3985] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3986] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3987] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3988] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3989] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3990] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3991] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3992] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[3993] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3994] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3995] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3996] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[3997] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[3998] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[3999] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4000] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4001] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4002] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4003] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4004] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4005] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4006] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4007] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4008] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4009] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4010] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4011] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4012] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4013] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4014] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4015] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4016] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4017] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4018] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4019] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4020] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4021] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4022] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4023] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4024] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4025] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4026] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4027] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4028] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4029] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4030] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4031] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]+H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]-J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]-J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]-J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]-J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]-J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4032] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]-J[44]-J[45]-J[46]-J[47]-J[48]-J[49]-J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4033] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]-J[43]+J[44]-J[45]-J[46]-J[47]-J[48]-J[49]+J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4034] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]-J[44]-J[45]-J[46]-J[47]-J[48]+J[49]-J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4035] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]-J[42]+J[43]+J[44]-J[45]-J[46]-J[47]-J[48]+J[49]+J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4036] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]-J[44]-J[45]-J[46]-J[47]+J[48]-J[49]-J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4037] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]-J[43]+J[44]-J[45]-J[46]-J[47]+J[48]-J[49]+J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4038] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]-J[44]-J[45]-J[46]-J[47]+J[48]+J[49]-J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4039] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]-J[41]+J[42]+J[43]+J[44]-J[45]-J[46]-J[47]+J[48]+J[49]+J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4040] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]-J[44]-J[45]-J[46]+J[47]-J[48]-J[49]-J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4041] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]-J[43]+J[44]-J[45]-J[46]+J[47]-J[48]-J[49]+J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4042] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]-J[44]-J[45]-J[46]+J[47]-J[48]+J[49]-J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4043] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]-J[42]+J[43]+J[44]-J[45]-J[46]+J[47]-J[48]+J[49]+J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4044] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]-J[44]-J[45]-J[46]+J[47]+J[48]-J[49]-J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4045] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]-J[43]+J[44]-J[45]-J[46]+J[47]+J[48]-J[49]+J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4046] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]-J[44]-J[45]-J[46]+J[47]+J[48]+J[49]-J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4047] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]-J[40]+J[41]+J[42]+J[43]+J[44]-J[45]-J[46]+J[47]+J[48]+J[49]+J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4048] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]-J[44]-J[45]+J[46]-J[47]-J[48]-J[49]-J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4049] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]-J[43]+J[44]-J[45]+J[46]-J[47]-J[48]-J[49]+J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4050] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]-J[44]-J[45]+J[46]-J[47]-J[48]+J[49]-J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4051] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]-J[42]+J[43]+J[44]-J[45]+J[46]-J[47]-J[48]+J[49]+J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4052] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]-J[44]-J[45]+J[46]-J[47]+J[48]-J[49]-J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4053] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]-J[43]+J[44]-J[45]+J[46]-J[47]+J[48]-J[49]+J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4054] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]-J[44]-J[45]+J[46]-J[47]+J[48]+J[49]-J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4055] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]-J[41]+J[42]+J[43]+J[44]-J[45]+J[46]-J[47]+J[48]+J[49]+J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4056] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]-J[44]-J[45]+J[46]+J[47]-J[48]-J[49]-J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4057] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]-J[43]+J[44]-J[45]+J[46]+J[47]-J[48]-J[49]+J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4058] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]-J[44]-J[45]+J[46]+J[47]-J[48]+J[49]-J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4059] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]-J[42]+J[43]+J[44]-J[45]+J[46]+J[47]-J[48]+J[49]+J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4060] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]-J[44]-J[45]+J[46]+J[47]+J[48]-J[49]-J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4061] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]-J[43]+J[44]-J[45]+J[46]+J[47]+J[48]-J[49]+J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4062] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]-J[44]-J[45]+J[46]+J[47]+J[48]+J[49]-J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4063] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]+H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]-J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]-J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]-J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]-J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]-J[39]+J[40]+J[41]+J[42]+J[43]+J[44]-J[45]+J[46]+J[47]+J[48]+J[49]+J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4064] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]-J[44]+J[45]-J[46]-J[47]-J[48]-J[49]-J[50]-J[51]-J[52]-J[53]-J[54]-J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4065] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]-J[43]+J[44]+J[45]-J[46]-J[47]-J[48]-J[49]+J[50]-J[51]-J[52]-J[53]-J[54]+J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4066] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]-J[44]+J[45]-J[46]-J[47]-J[48]+J[49]-J[50]-J[51]-J[52]-J[53]+J[54]-J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4067] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]-J[42]+J[43]+J[44]+J[45]-J[46]-J[47]-J[48]+J[49]+J[50]-J[51]-J[52]-J[53]+J[54]+J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4068] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]-J[44]+J[45]-J[46]-J[47]+J[48]-J[49]-J[50]-J[51]-J[52]+J[53]-J[54]-J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4069] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]-J[43]+J[44]+J[45]-J[46]-J[47]+J[48]-J[49]+J[50]-J[51]-J[52]+J[53]-J[54]+J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4070] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]-J[44]+J[45]-J[46]-J[47]+J[48]+J[49]-J[50]-J[51]-J[52]+J[53]+J[54]-J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4071] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]-J[41]+J[42]+J[43]+J[44]+J[45]-J[46]-J[47]+J[48]+J[49]+J[50]-J[51]-J[52]+J[53]+J[54]+J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4072] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]-J[44]+J[45]-J[46]+J[47]-J[48]-J[49]-J[50]-J[51]+J[52]-J[53]-J[54]-J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4073] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]-J[43]+J[44]+J[45]-J[46]+J[47]-J[48]-J[49]+J[50]-J[51]+J[52]-J[53]-J[54]+J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4074] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]-J[44]+J[45]-J[46]+J[47]-J[48]+J[49]-J[50]-J[51]+J[52]-J[53]+J[54]-J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4075] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]-J[42]+J[43]+J[44]+J[45]-J[46]+J[47]-J[48]+J[49]+J[50]-J[51]+J[52]-J[53]+J[54]+J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4076] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]-J[44]+J[45]-J[46]+J[47]+J[48]-J[49]-J[50]-J[51]+J[52]+J[53]-J[54]-J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4077] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]-J[43]+J[44]+J[45]-J[46]+J[47]+J[48]-J[49]+J[50]-J[51]+J[52]+J[53]-J[54]+J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4078] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]-J[44]+J[45]-J[46]+J[47]+J[48]+J[49]-J[50]-J[51]+J[52]+J[53]+J[54]-J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4079] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]+H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]-J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]-J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]-J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]-J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]-J[40]+J[41]+J[42]+J[43]+J[44]+J[45]-J[46]+J[47]+J[48]+J[49]+J[50]-J[51]+J[52]+J[53]+J[54]+J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4080] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]-J[44]+J[45]+J[46]-J[47]-J[48]-J[49]-J[50]+J[51]-J[52]-J[53]-J[54]-J[55]-J[56]-J[57]-J[58]-J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4081] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]-J[43]+J[44]+J[45]+J[46]-J[47]-J[48]-J[49]+J[50]+J[51]-J[52]-J[53]-J[54]+J[55]-J[56]-J[57]-J[58]+J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4082] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]-J[44]+J[45]+J[46]-J[47]-J[48]+J[49]-J[50]+J[51]-J[52]-J[53]+J[54]-J[55]-J[56]-J[57]+J[58]-J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4083] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]-J[42]+J[43]+J[44]+J[45]+J[46]-J[47]-J[48]+J[49]+J[50]+J[51]-J[52]-J[53]+J[54]+J[55]-J[56]-J[57]+J[58]+J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4084] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]-J[44]+J[45]+J[46]-J[47]+J[48]-J[49]-J[50]+J[51]-J[52]+J[53]-J[54]-J[55]-J[56]+J[57]-J[58]-J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4085] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]-J[43]+J[44]+J[45]+J[46]-J[47]+J[48]-J[49]+J[50]+J[51]-J[52]+J[53]-J[54]+J[55]-J[56]+J[57]-J[58]+J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4086] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]-J[44]+J[45]+J[46]-J[47]+J[48]+J[49]-J[50]+J[51]-J[52]+J[53]+J[54]-J[55]-J[56]+J[57]+J[58]-J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4087] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]+H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]-J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]-J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]-J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]-J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]-J[41]+J[42]+J[43]+J[44]+J[45]+J[46]-J[47]+J[48]+J[49]+J[50]+J[51]-J[52]+J[53]+J[54]+J[55]-J[56]+J[57]+J[58]+J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4088] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]-J[44]+J[45]+J[46]+J[47]-J[48]-J[49]-J[50]+J[51]+J[52]-J[53]-J[54]-J[55]+J[56]-J[57]-J[58]-J[59]-J[60]-J[61]-J[62]+J[63]+J[64]+J[65] - logZ )
    Pout[4089] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]-J[43]+J[44]+J[45]+J[46]+J[47]-J[48]-J[49]+J[50]+J[51]+J[52]-J[53]-J[54]+J[55]+J[56]-J[57]-J[58]+J[59]-J[60]-J[61]+J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4090] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]-J[44]+J[45]+J[46]+J[47]-J[48]+J[49]-J[50]+J[51]+J[52]-J[53]+J[54]-J[55]+J[56]-J[57]+J[58]-J[59]-J[60]+J[61]-J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4091] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]+H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]-J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]-J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]-J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]-J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]-J[42]+J[43]+J[44]+J[45]+J[46]+J[47]-J[48]+J[49]+J[50]+J[51]+J[52]-J[53]+J[54]+J[55]+J[56]-J[57]+J[58]+J[59]-J[60]+J[61]+J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4092] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]-J[44]+J[45]+J[46]+J[47]+J[48]-J[49]-J[50]+J[51]+J[52]+J[53]-J[54]-J[55]+J[56]+J[57]-J[58]-J[59]+J[60]-J[61]-J[62]-J[63]-J[64]+J[65] - logZ )
    Pout[4093] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]+H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]-J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]-J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]-J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]-J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]-J[43]+J[44]+J[45]+J[46]+J[47]+J[48]-J[49]+J[50]+J[51]+J[52]+J[53]-J[54]+J[55]+J[56]+J[57]-J[58]+J[59]+J[60]-J[61]+J[62]-J[63]+J[64]-J[65] - logZ )
    Pout[4094] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]+H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]-J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]-J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]-J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]-J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]-J[44]+J[45]+J[46]+J[47]+J[48]+J[49]-J[50]+J[51]+J[52]+J[53]+J[54]-J[55]+J[56]+J[57]+J[58]-J[59]+J[60]+J[61]-J[62]+J[63]-J[64]-J[65] - logZ )
    Pout[4095] = exp( -H[0]-H[1]-H[2]-H[3]-H[4]-H[5]-H[6]-H[7]-H[8]-H[9]-H[10]-H[11]+J[0]+J[1]+J[2]+J[3]+J[4]+J[5]+J[6]+J[7]+J[8]+J[9]+J[10]+J[11]+J[12]+J[13]+J[14]+J[15]+J[16]+J[17]+J[18]+J[19]+J[20]+J[21]+J[22]+J[23]+J[24]+J[25]+J[26]+J[27]+J[28]+J[29]+J[30]+J[31]+J[32]+J[33]+J[34]+J[35]+J[36]+J[37]+J[38]+J[39]+J[40]+J[41]+J[42]+J[43]+J[44]+J[45]+J[46]+J[47]+J[48]+J[49]+J[50]+J[51]+J[52]+J[53]+J[54]+J[55]+J[56]+J[57]+J[58]+J[59]+J[60]+J[61]+J[62]+J[63]+J[64]+J[65] - logZ )

    Pout = Pout[::-1]
    return(Pout)
